# -*- coding: cp1250 -*-
#===============================================================================
#=           Zakladni sada funkci pro praci s Lindenmayerovy systemy           =
#=                            Petr Schreiber, 2008                             =
#=                                    v1.0                                     =
#=                                                                             =
#=           Kod byl navrzen tak, aby byl prehledny, ne nutne rychly           =
#=                                                                             =
#=                               Mozne problemy:                               =
#=          velke mnozstvi iteraci muze vyustit v generovani stringu           =
#=            obrich rozmeru, coz muze prinest problemy s rychlosti            =
#=                             i pametovymi naroky                             =
#===============================================================================
#= Minimalni priklad pouziti ( 2 iterace Kochovy krivky )                      =
#= Koch = TLindenmayerSystem('F')                                              =
#= Koch.addVariable('F', 'F+F-F-F+F')                                          =
#= Koch.addConstant('+')                                                       =
#= Koch.addConstant('-')                                                       =
#=                                                                             =
#= Koch.iterate(2)                                                             =
#= print Koch.LString                                                          =
#===============================================================================
  
class TLindenmayerSystem:

  # Konstruktor bere jako vstup pocatecni retezec
  def __init__(self, startString ):
    self.LString = startString
    self.__Rules = []

  # Pridavani promenne i jeji transformace    
  def addVariable(self, aSymbol, aPattern):
    if (len(aPattern) == 1):   # \n
      aPattern = aSymbol    
      print ("[i] Varovani: Pokud nechcete aby byl symbol nahrazovan, pouzijte metodu addConstant()")
      pass      
    self.__Rules.append([aSymbol, aPattern])

  # Pridavani konstanty
  def addConstant(self, aSymbol):      
    self.__Rules.append([aSymbol, aSymbol])
    
  # Metoda pro provedeni 1..N iteraci    
  def iterate(self, aTimes=1):

    # Iterace
    for i in range(0, aTimes):
      tempString = self.LString
      self.LString = ''
    
      # Pro kazdy znak ...
      for ch in tempString:
        foundMatch = False
        
        # ... je aplikovano dostupne pravidlo
        for rule in self.__Rules:
          if (ch == rule[0]):
            self.LString += rule[1]
            foundMatch = True                   
            break  
             
        # Neni-li nalezeno, znak se nijak nezmeni
        if (foundMatch == False):        
          self.LString += ch
   