# -*- coding: cp1250 -*-
#===============================================================================
#=                                  Vrstva 1                                   =
#=            Zajistuje transformaci a definici zakladnich objektu             =
#=                                                                             =
#=                               Petr Schreiber                                =
#=                                    v1.0                                     =
#===============================================================================
from __future__ import division  # Nutne pro floating point deleni
from XODEGenerator_L0 import *

#===============================================================================
#=                                Transformace                                 =
#===============================================================================
class auxTransform:

  def __init__(self):
  
    # Zasobnik transformaci pozice
    self.__posStk = {}                  # Samotny zasobnik
    self.__posStkLvl = 0                # Pozice v nem
    
    # 0 = x, 1 = y, 2 = z
    self.__posStk[self.__posStkLvl, 0]       = 0  # Data pro uroven 0
    self.__posStk[self.__posStkLvl, 1]       = 0
    self.__posStk[self.__posStkLvl, 2]       = 0
  
  # Nastavi absolutni pozici 
  def posSet(self, x=0, y=0, z=0 ):
  

        
    if self.__posStkLvl > 0:
      self.__posStk[self.__posStkLvl, 0] = self.__posStk[self.__posStkLvl-1, 0]+x 
      self.__posStk[self.__posStkLvl, 1] = self.__posStk[self.__posStkLvl-1, 1]+y   
      self.__posStk[self.__posStkLvl, 2] = self.__posStk[self.__posStkLvl-1, 2]+z  
    else:    
      self.__posStk[self.__posStkLvl, 0] = x 
      self.__posStk[self.__posStkLvl, 1] = y   
      self.__posStk[self.__posStkLvl, 2] = z
        
  # Prida k aktualni pozici vektor  
  def posAdd(self,  x, y, z ):
    self.__posStk[self.__posStkLvl,0] += x
    self.__posStk[self.__posStkLvl,1] += y   
    self.__posStk[self.__posStkLvl,2] += z    
  
  # Zalohuje zasobnik
  def posPush(self):
    self.__posStkLvl += 1
    self.__posStk[self.__posStkLvl, 0]       = self.__posStk[self.__posStkLvl-1, 0]
    self.__posStk[self.__posStkLvl, 1]       = self.__posStk[self.__posStkLvl-1, 1]
    self.__posStk[self.__posStkLvl, 2]       = self.__posStk[self.__posStkLvl-1, 2]
  
  # Obnovi puvodni hodnoty  
  def posPop(self):  
    self.__posStkLvl -= 1
    if (self.__posStkLvl < 0):
      print "[!] Chyba : Podteceni zasobniku"
      self.__posStkLvl = 0    
  
  # Vraci vektor aktualni pozice    
  def posGet(self):
    return self.__posStk[self.__posStkLvl, 0], self.__posStk[self.__posStkLvl, 1], self.__posStk[self.__posStkLvl, 2]             

class XODEGenerator_L1(XODEGenerator_L0, auxTransform):

  def __init__(self):    
    XODEGenerator_L0.__init__(self)
    auxTransform.__init__(self)
    
    self.objTrimesh = []                            # List, ktery obsahuje nactene trimeshe
    self.bodyCount  = 0                             # Nepojmenovana bodies budou autocislovana
    self.geomCount  = 0                             # Nepojmenovana geoms budou autocislovana
    
    # Nastaveni zdi
    self.wallWidth = 0.2
    self.wallHeight= 2.5
    
    # Nastaveni RGB barvy
    self.globalColor = [192, 192, 192]     
    
    self.wantTransparency = False
  
  # "PRIVATE"
  def bodyBegin(self, name, angles=[0, 0, 0], mass=1 ):
    self.fileWriteLn("<body name=\""+name+"\">")
    x, y, z = self.posGet()
    self.fileWriteLn("<transform><position x=\""+str(x)+"\" y=\""+str(y)+"\" z=\""+str(z)+"\" /><rotation><euler x=\""+str(angles[0])+"\" y=\""+str(angles[1])+"\" z=\""+str(angles[2])+"\" aformat=\"degrees\"/></rotation></transform>\n")      
    self.fileWriteLn("<mass><adjust total=\""+str(mass)+"\" /></mass>")
      
  def bodyEnd(self ):
    self.fileWriteLn("</body>")  

  def geomBegin(self, name ):
    if (self.wantTransparency == True):
      name += "--transparent"
    self.fileWriteLn("<geom name=\""+name+","+str(self.globalColor[0])+","+str(self.globalColor[1])+","+str(self.globalColor[2])+"\">")  

  def geomEnd(self ):
    self.fileWriteLn("</geom>")  

  def __posRot(self, angles=[0, 0, 0]):
    x, y, z = self.posGet()  
    if (len(angles) != 3):
      angles=[0, 0, 0]
    
    self.fileWriteLn("<transform><position x=\""+str(x)+"\" y=\""+str(y)+"\" z=\""+str(z)+"\" /><rotation><euler x=\""+str(angles[0])+"\" y=\""+str(angles[1])+"\" z=\""+str(angles[2])+"\" aformat=\"degrees\"/></rotation></transform>\n")

  # "PUBLIC"
  # Nastaveni barvy pro objekty ( skladajici se z ) GEOM
  def clrSet(self, rgb):
    self.globalColor = rgb

  def clrGet(self):
    return self.globalColor 
    
  def useTransparency(self, state):
    self.wantTransparency = state    
     
  # Provede konverzi OBJ na Trimesh a data ulozi do listu  
  def preloadOBJ2Trimesh(self, fName):
    
    jmeno_fOBJ = fName
    jmeno_fTRI = jmeno_fOBJ.split('.')[0]+'.tri'
    
    # Ziskam radky z OBJ souboru
    handle_fOBJ = file(jmeno_fOBJ, 'r')
    linesOBJ = handle_fOBJ.readlines()
    handle_fOBJ.close()
    
    linesOut = []
    
    # Zacnu budovat trimesh    
    foundFace = 0
    hadToConvert = 0
    triangles = 0  
    
    linesOut.append("<trimesh><vertices>\n")  
    for line in linesOBJ:
      line = line.strip()           # odstani CR, LF a prebytecne mezery
      if line.startswith('v '):
        vertex = line.split(' ')     
        linesOut.append("<v x=\""+vertex[1]+"\" y=\""+vertex[2]+"\" z=\""+vertex[3]+"\" />\n")    
      if line.startswith('f '):
        if foundFace == 0:
          foundFace = 1
          linesOut.append("</vertices>\n<triangles>\n")
    
        vertex = line.split(' ')
        
        # Definice polygonu muze obsahovat i odkaz na texturocvaci koordinaty, ty byvaji oddelen lomitkem
        triangles += 1
        linesOut.append("<t ia=\""+vertex[1].split("/")[0]+"\" ib=\""+vertex[2].split("/")[0]+"\" ic=\""+vertex[3].split("/")[0]+"\" />\n")
        
        # je treba rozlozit na tris
        if len(vertex) > 4:
          hadToConvert = 1
          for i in range(3, len(vertex)-1):
            triangles += 1
            linesOut.append("<t ia=\""+vertex[1].split("/")[0]+"\" ib=\""+vertex[i].split("/")[0]+"\" ic=\""+vertex[i+1].split("/")[0]+"\" />\n")        
    linesOut.insert(0, "\n<geom name=\""+jmeno_fOBJ.split('.')[0]+", 255, 128, 0, "+str(triangles)+"\">\n")  
    linesOut.append("</triangles></trimesh>\n</geom>\n")
    self.objTrimesh.append(linesOut)  
      
    return len(self.objTrimesh )-1
      
  # Vlozi trimesh do souboru   
  def addTrimesh(self, handle, weight=0, name='', orientation=[0, 0, 0]):
    
    # Vlozi pozici, a zapise do souboru
    if handle > len(self.objTrimesh):
      print "[!] Chyba : Neplatny handle modelu"
    else:  
      x, y, z = self.posGet()
      if (weight > 0):
        if (len(name) == 0):
          self.bodyCount += 1
          name == 'bTrimesh'+str(self.bodyCount)
           
        self.bodyBegin(name, angles=orientation, mass=weight)
        self.fileWriteMultiLine(self.objTrimesh[handle])
        self.bodyEnd()
      else:
        self.objTrimesh[handle].insert(1, "<transform><position x=\""+str(x)+"\" y=\""+str(y)+"\" z=\""+str(z)+"\" /><rotation><euler x=\""+str(orientation[0])+"\" y=\""+str(orientation[1])+"\" z=\""+str(orientation[2])+"\" aformat=\"degrees\"/></rotation></transform>\n")
        self.fileWriteMultiLine(self.objTrimesh[handle])
        self.objTrimesh[handle].pop(1)
        
  # Vlozi kouli do souboru      
  def addSphere(self, weight, radius, name=''):
    if (name == ''):
      self.bodyCount += 1
      self.geomCount += 1
      bname = 'bSphere'+str(self.bodyCount)
      gname = 'gSphere'+str(self.geomCount)
    else:
      bname = name
      gname = 'g'+name  
      
    if (weight > 0):    # Jedna se o pohyblive teleso
      self.bodyBegin(bname, mass=weight)
      self.geomBegin(gname)
      self.fileWriteLn("<sphere radius=\""+str(radius)+"\"/>")
      self.geomEnd()
      self.bodyEnd()
    else:               # Jedna se o staticke teleso / prekazku
      
      self.geomBegin(gname)
      self.__posRot()
      self.fileWriteLn("<sphere radius=\""+str(radius)+"\"/>")
      self.geomEnd()  
      
  # Vlozi krychli do souboru    
  def addBox(self, weight, sx, sy, sz, rot=[0,0,0], name=''):
    if (name == ''):
      self.bodyCount += 1
      self.geomCount += 1
      bname = 'bBox'+str(self.bodyCount)
      gname = 'gBox'+str(self.geomCount)
    else:
      bname = name
      gname = 'g'+name  
      
    if (weight > 0):    # Jedna se o pohyblive teleso
      self.bodyBegin(bname, mass=weight)
      self.geomBegin(gname)
      self.fileWriteLn("<box sizex=\""+str(sx)+"\" sizey=\""+str(sy)+"\" sizez=\""+str(sz)+"\"/>")
      self.geomEnd()
      self.bodyEnd()
    else:               # Jedna se o staticke teleso / prekazku      
      self.geomBegin(gname)
      self.__posRot(angles=rot)
      self.fileWriteLn("<box sizex=\""+str(sx)+"\" sizey=\""+str(sy)+"\" sizez=\""+str(sz)+"\"/>")
      self.geomEnd()           
  
  # Vlozi valec do souboru    
  def addCylinder(self, weight, radius, length, dir, name=''):
    if (name == ''):
      self.bodyCount += 1
      self.geomCount += 1
      bname = 'bCylinder'+str(self.bodyCount)
      gname = 'gCylinder'+str(self.geomCount)
    else:
      bname = name
      gname = 'g'+name  
    if (dir == 1):
      rot = [0, -90, 0]
    elif (dir == 2):
      rot = [-90, 0, 0]
    else:
      rot = [0, 0, 0]  
      
    if (weight > 0):    # Jedna se o pohyblive teleso
      self.bodyBegin(bname, mass=weight, angles=rot)
      self.geomBegin(gname)
      self.fileWriteLn("<cylinder radius=\""+str(radius)+"\" length=\""+str(length)+"\" />")
      self.geomEnd()
      self.bodyEnd()
    else:               # Jedna se o staticke teleso / prekazku      
      self.geomBegin(gname)
      self.__posRot(angles=rot)
      self.fileWriteLn("<cylinder radius=\""+str(radius)+"\" length=\""+str(length)+"\" />")
      
      self.geomEnd()    
      
   # Vlozi podlahu na pozici 0, 0, 0
  def putFloor(self):
    self.geomBegin("simFloor")
  
    self.fileWriteLn("<plane a=\"0\" b=\"1\" c=\"0\" d=\"0\" />")
    self.geomEnd()        
   
   # ----------------- SPECIALNI PRIPADY ---------------------   
  def addBoxCBLC(self, weight, sx, sy, sz, name=''):
    self.posPush()
    self.posAdd(sx/2, sy/2, sz/2)
    self.addBox(weight, sx, sy, sz, [0,0,0],name)
    self.posPop()    

  def addColumn(self, sx, sy, sz, rot=[0,0,0], name=''):
    self.posPush()
    self.posAdd(0, sy/2, 0)
    self.addBox(0, sx, sy, sz, rot, name)
    self.posPop()
        
  def setWallParams(self, width, height):
    self.wallWidth = width
    self.wallHeight= height

  def getWallParams(self):
    return self.wallWidth, self.wallHeight
      
  def __Sign( self, number):
    return number/abs(number)
        
  def addWall(self, len, dir, movposToEnd=0):    
    
    self.posPush()
    if (abs(dir) == 1):      
      len = len * self.__Sign(dir)
      self.posAdd(len/2, self.wallHeight/2, self.wallWidth/2)
      len = abs(len)
      self.addBox(0, len, self.wallHeight, self.wallWidth, name)
    elif (abs(dir) == 3):
      len = len * self.__Sign(dir)
      self.posAdd(self.wallWidth/2, self.wallHeight/2, len/2)
      len = abs(len)
      self.addBox(0, self.wallWidth, self.wallHeight, len, name)
    else:
      print ("[!] Chyba : Smer neni podporovan, pouzijte 1 nebo 3")    
          
    self.posPop()
    
    # Ne zcela spolehlive  
    if (movposToEnd == True):
      if (abs(dir) == 1):    
        self.posAdd(len * self.__Sign(dir), 0, 0) 
      elif (abs(dir) == 3):    
        self.posAdd(0, 0, len * self.__Sign(dir)) 
    