# -*- coding: cp1250 -*-
#===============================================================================
#=                                  Vrstva 3                                   =
#=                 Obsahuje generatory velkych casti prostredi                 =
#=                                                                             =
#=                               Petr Schreiber                                =
#=                                    v1.0                                     =
#===============================================================================
from __future__ import division  # Nutne pro floating point deleni
from XODEGenerator_L2 import *
import random

class XODEGenerator_L3(XODEGenerator_L2):

  def __init__(self):
    XODEGenerator_L2.__init__(self)
    
  def __rnd( self, aFrom, aTo ):
    return random.randint( aFrom, aTo )
    
  def addRooms(self, seed=1, xSize=20, zSize=20, roomsX=5, roomsZ=5, wantChair=True, wantTable=True, wantWardrobe=True):
    random.seed(seed)
    originalRGB = self.clrGet() 
    self.posPush()
    
    doorWidth = 0.5
    
    wallWidth, wallHeight = self.getWallParams()
    
    # Pouziji rooms jako 2-rozmerne pole, ovsem s pomoci dictionary
    rooms = {}
    
    # Typy mistnosti
    ROOM_B1  = 1    # Velka
    ROOM_H2  = 2    # 2 horizontalni
    ROOM_V2  = 3    # 2 vertikalni
    ROOM_S4  = 4    # 4 male, stejne velke
  
    # Vygeneruje "pole", ktere bude obsahovat nahodne mistnosti  
    for x in range(1, roomsX+1):                      # ekvivalent FOR x = 1 TO roomsX
      for y in range(1, roomsZ+1):                    # ekvivalent FOR y = 1 TO roomsZ          
        #if (x == 1) and(y == 1):
        #  rooms[x, y] = self.__rnd(ROOM_B1, ROOM_V2)
        #else:
        rooms[x, y] = self.__rnd(ROOM_B1, ROOM_S4 )  
         
    # Tvorba mistnosti na zaklade udaju z pole    
    for x in range(1, roomsX+1):                      # ekvivalent FOR x = 1 TO roomsX
      for y in range(1, roomsZ+1):                    # ekvivalent FOR y = 1 TO roomsZ      

        # Zdi bez dveri
        self.clrSet(originalRGB) 
             
        # Python bohuzel nema SWITCH / SELECT CASE        
        if (rooms[x, y] == ROOM_H2):           
          self.posSet((x-1)*xSize/roomsX+wallWidth, 0, (y-1)*zSize/roomsZ+zSize/roomsZ/2)
          self.addBoxCBLC(0, xSize/roomsX-wallWidth, wallHeight, wallWidth)
          
        elif (rooms[x, y] == ROOM_V2):        
          self.posSet( (x-1)*xSize/roomsX+xSize/roomsX/2, 0, (y-1)*zSize/roomsZ+wallWidth )
          self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ-wallWidth)
          
        elif (rooms[x, y] == ROOM_S4):  
          # h1        
          self.posSet( (x-1)*xSize/roomsX+wallWidth, 0, (y-1)*zSize/roomsZ+zSize/roomsZ/2 )
          self.addBoxCBLC(0, xSize/roomsX/2-wallWidth, wallHeight, wallWidth)
          self.posAdd( xSize/roomsX/2, 0, 0 )                
          self.addBoxCBLC(0, xSize/roomsX/2-wallWidth, wallHeight, wallWidth)                
          # v1        
          self.posSet( (x-1)*xSize/roomsX+xSize/roomsX/2, 0, (y-1)*zSize/roomsZ+wallWidth )
          self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/2-wallWidth)
          self.posAdd( 0, 0, zSize/roomsZ/2 )                                
          self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/2-wallWidth)                
                    
        # -- Vertikalni prepazky
        #self.clrSet([0, 255, 255)  
        if x > 1:
          
          # -- Pokud je 2x delena ve vertikalnim smeru libovolna leva/prava
          if ((rooms[x, y] % 2) == 0) or (rooms[x-1, y] % 2 == 0):
            #self.clrSet([255, 128, 0)                      
            self.posSet( (x-1)*xSize/roomsX, 0, (y-1)*zSize/roomsZ+wallWidth )
            self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/4-wallWidth-doorWidth)
            self.posAdd(0, 0, zSize/roomsZ/4+doorWidth)
                
            if (rooms[x-1, y] == ROOM_S4) or (rooms[x-1, y] == ROOM_H2):
              #self.clrSet([255, 128, 255)
              self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/4-2*doorWidth+zSize/roomsZ/4-wallWidth)                  
              self.posAdd(0, 0, zSize/roomsZ/2)
            else:
              #self.clrSet([255, 255, 255)
              self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/2-doorWidth*2)                                     
              self.posAdd(0, 0, zSize/roomsZ/2)                    
  
            self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/4-doorWidth-wallWidth)                  
                 
          elif ((rooms[x, y] % 2) <> 0 ) and((rooms[x-1, y] % 2) <> 0):
            #self.clrSet([0, 128, 255)
            
            self.posSet( (x-1)*xSize/roomsX, 0, (y-1)*zSize/roomsZ+wallWidth )
            self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/2-wallWidth-doorWidth)
            self.posAdd(0, 0, zSize/roomsZ/2+doorWidth)
            self.addBoxCBLC(0, wallWidth, wallHeight, zSize/roomsZ/2-wallWidth-doorWidth)                  
                           
          # -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
        
        #self.clrSet([0, 255, 0)
        # -- Horizontalni prepazky          
        if (y > 1):
          # -- Pokud je 2x delena v horiz smeru libovolna horni dolni
          if (rooms[x, y+0] == ROOM_V2) or (rooms[x, y+0] == ROOM_S4) or \
             (rooms[x, y-1] == ROOM_V2) or (rooms[x, y-1] == ROOM_S4):
                        
            self.posSet( (x-1)*xSize/roomsX+wallWidth, 0, (y-1)*zSize/roomsZ )
            self.addBoxCBLC(0, xSize/roomsX/4-wallWidth-doorWidth, wallHeight, wallWidth)
            self.posAdd(xSize/roomsX/4+doorWidth, 0, 0)
                
            if (rooms[x, y-1] == ROOM_S4) or (rooms[x, y-1] == ROOM_H2):
              self.addBoxCBLC(0, xSize/roomsX/4-2*doorWidth+xSize/roomsX/4-wallWidth, wallHeight, wallWidth)                  
              self.posAdd(xSize/roomsX/4-doorWidth, 0, 0)                                
              self.posAdd(xSize/roomsX/4+doorWidth, 0, 0)
            else:
              self.addBoxCBLC(0, xSize/roomsX/2-doorWidth*2, wallHeight, wallWidth)                                     
              self.posAdd(xSize/roomsX/2, 0, 0)                    
    
            self.addBoxCBLC(0, xSize/roomsX/4-doorWidth-wallWidth, wallHeight, wallWidth)                  
              
          else:
            self.posSet( (x-1)*xSize/roomsX+wallWidth, 0, (y-1)*zSize/roomsZ )
            self.addBoxCBLC(0, xSize/roomsX/2-wallWidth-doorWidth, wallHeight, wallWidth)
            self.posAdd(xSize/roomsX/2+doorWidth, 0, 0)
            self.addBoxCBLC(0, xSize/roomsX/2-wallWidth-doorWidth, wallHeight, wallWidth)
                            
        # -- Nabytek 
  
        if (x != 1) or (y != 1):
          if (rooms[x, y] == ROOM_B1):  
              
            self.clrSet([128, 0, 0])
            self.posSet((x-1)*xSize/roomsX+xSize/roomsX/2, 0, (y-1)*zSize/roomsZ+zSize/roomsZ/2)
            
            if (wantTable==True):
              temp = self.__rnd(0, 1)
              if (temp == 0):
                self.addTable(1, 0.7, 2, self.__rnd(0, 4), 0.1, 'box', 0.1, 0.9, 1.8)
              else:
                self.addTable(1, 0.7, 1, self.__rnd(0, 4), 0.1, 'cylinder', 0.1, 0.5, 0.5)

            if (wantChair==True):                       
              self.clrSet([255, 128, 0])  
              for i in range(0, 4):  # = 0..3
                if self.__rnd(0,1) == 1:
                
                  self.posPush()
                  # Stolek se zidlemi
                  uhel = i * 90
                  
                  self.posSet(-sin(radians(uhel)), 0, -cos(radians(uhel))*1.5)
                  self.addChair(0.35, 1, 0.40, i, 0.05, 0.4, 0.05, 0.2, 0.3)
     
                  self.posPop()
          if (wantWardrobe==True):                                  
            if (y == 1) or (rooms[x, y-1] == ROOM_B1) or (rooms[x, y-1] == ROOM_H2):
      
              self.clrSet([self.__rnd(128,255), self.__rnd(128,255), self.__rnd(128,255)]) 
              self.posSet((x-1)*xSize/roomsX+0.625, 0, (y-1)*zSize/roomsZ+0.5)
              self.addWardrobe(1, self.__rnd(1, 2), 0.5, \
                                  0, \
                                  0.05, \
                                  self.__rnd(0, 1), self.__rnd(0, 1), self.__rnd(0, 1), \
                                  self.__rnd(5, 8), 0.1, \
                                  self.__rnd(0, 1), self.__rnd(0, 135), self.__rnd(0, 135))
  
              self.clrSet([self.__rnd(128,255), self.__rnd(128,255), self.__rnd(128,255)]) 
              self.posSet((x-1)*xSize/roomsX+0.625+xSize/roomsX/2+doorWidth, 0, (y-1)*zSize/roomsZ+0.5)
              self.addWardrobe(1, self.__rnd(1, 2), 0.5, \
                                  0, \
                                  0.05, \
                                  self.__rnd(0, 1), self.__rnd(0, 1), self.__rnd(0, 1), \
                                  self.__rnd(5, 8), 0.1, \
                                  self.__rnd(0, 1), self.__rnd(0, 135), self.__rnd(0, 135))
  
            if (y == roomsZ) or (rooms[x, y+1] == ROOM_B1) or (rooms[x, y+1] == ROOM_H2):
      
              self.clrSet([self.__rnd(128,255), self.__rnd(128,255), self.__rnd(128,255)]) 
              self.posSet((x-1)*xSize/roomsX+0.625, 0, (y)*zSize/roomsZ-0.5)
              self.addWardrobe(1, self.__rnd(1, 2), 0.5, \
                                  2, \
                                  0.05, \
                                  self.__rnd(0, 1), self.__rnd(0, 1), self.__rnd(0, 1), \
                                  self.__rnd(5, 8), 0.1, \
                                  self.__rnd(0, 1), self.__rnd(0, 135), self.__rnd(0, 135))
  
              self.clrSet([self.__rnd(128,255), self.__rnd(128,255), self.__rnd(128,255)]) 
              self.posSet((x-1)*xSize/roomsX+0.625+xSize/roomsX/2+doorWidth, 0, (y)*zSize/roomsZ-0.5)
              self.addWardrobe(1, self.__rnd(1, 2), 0.5, \
                                  2, \
                                  0.05, \
                                  self.__rnd(0, 1), self.__rnd(0, 1), self.__rnd(0, 1), \
                                  self.__rnd(5, 8), 0.1, \
                                  self.__rnd(0, 1), self.__rnd(0, 135), self.__rnd(0, 135))
 
    # Hranice :)
    self.clrSet([255, 255, 255])
    self.useTransparency(True)
    self.posSet(wallWidth, 0, 0)  
    self.addBoxCBLC(0, xSize - wallWidth,wallHeight,wallWidth)
    
    self.posSet(xSize, 0, 0)  
    self.addBoxCBLC(0, wallWidth, wallHeight, zSize )
    
    self.posSet(0, 0, 0)  
    self.addBoxCBLC(0, wallWidth, wallHeight, zSize+wallWidth)
    
    self.posSet(wallWidth, 0, zSize)
    self.addBoxCBLC(0, xSize , wallHeight, wallWidth)
    self.useTransparency(False)
    
    
    self.posPop()        
    self.clrSet(originalRGB)
    # Pripadne pridat skrine pos stranaaaach  
  
  def addLSystemWalls(self, LSystem, length, doorWidth, angle):
    singlePart = (length - doorWidth)/2
    singlePartH= singlePart/2    
    self.posPush()
    
    WIDTH, HEIGHT = self.getWallParams()
    
    LAngle = 0
    for ch in LSystem:
    
      if ch == 'F':
        
        self.posAdd( cos(radians(LAngle))*singlePartH,0,sin(radians(LAngle))*singlePartH)
        self.addBox(0, singlePart, HEIGHT, WIDTH, [0, LAngle, 0])
        self.posAdd( cos(radians(LAngle))*singlePartH,0,sin(radians(LAngle))*singlePartH)
        
        self.posAdd( cos(radians(LAngle))*(doorWidth),0,sin(radians(LAngle))*(doorWidth))
        self.posAdd( cos(radians(LAngle))*singlePartH,0,sin(radians(LAngle))*singlePartH)
        self.addBox(0, singlePart, HEIGHT, WIDTH, [0, LAngle, 0])
        self.posAdd( cos(radians(LAngle))*singlePartH,0,sin(radians(LAngle))*singlePartH)
        
      elif ch == '+':
        LAngle += angle
        
      elif ch == '-':
        LAngle -= angle

      elif ch == '[':
        self.posPush()
        
      elif ch == ']':
        self.posPop()
        
    self.posPop()
    