#===============================================================================
#=                            Soucast vzoroskriptu                             =
#=                                     v5                                      =
#===============================================================================
## obsahuje funkce ktere vytvari jednotiva telesa
## v0.3 - 2008.02.22 - odstranena duplicita zadavani hustoty i hmotnosti
## 2008.03.26 
#- funkce vraci i geom
#- geom v sobe drzi charakteristicke vlastnosti, body uz ne
 
import ode

def create_sphere (space,world,polomer,hmotnost, red = 128, green = 128, blue = 128):
  #1-koule
    body = ode.Body(world)          # vytvorim teleso ve svete, world je globalni promena
    M = ode.Mass()   
    M.setSphereTotal(hmotnost,polomer)
    body.setMass(M)                 # nastavim hmotu telesa

    #kvuli kolizim nastavim GEOMETRII pro dane teleso
    geom = ode.GeomSphere(space, polomer)
    geom.setBody(body)

    # do body pridam vlastnosti nutne pro vykresleni
    # pozn.: body nema vlastnost radius,shape ani color - timto je pridam a nastavim
    geom.shape = "sphere"           # nastavim TVAR pro dane teleso, pozdejsi zjisteni o jake teleso se jedna je mozne
                                    # provest i komplikovanym dotazem, timto by jsme si meli usetrit cas
    geom.color = [red,green,blue]              # tuto vlastnost pouziji pri vykreslovani pro urceni barvy
    geom.radius = polomer           # rozmery
    
    return body, geom

def create_box (space,world,hranaX,hranaY,hranaZ,hmotnost, red = 128, green = 128, blue = 128):
  #2-kvadr
    body = ode.Body(world)
    M = ode.Mass()
    M.setBoxTotal(hmotnost,hranaX,hranaY,hranaZ)    
    body.setMass(M)

    geom = ode.GeomBox(space,(hranaX,hranaY,hranaZ))   #ve kterem svete jsou kolize, a jak je teleso velke
    geom.setBody(body)
    geom.shape = "box"
    geom.color = [red,green,blue]
    geom.boxsize = (hranaX,hranaY,hranaZ)

    return body, geom

def create_cylinder (space,world,polomer,vyska,smer,hmotnost, red = 128, green = 128, blue = 128):
  #3-valec  
    body = ode.Body(world)
    M = ode.Mass()
    M.setCylinderTotal(hmotnost,smer,polomer,vyska) #density,direction(1(x),2(y),nebo 3(z)),radius,delka
    body.setMass(M)

    geom = ode.GeomCylinder(space,polomer,vyska)                  
    geom.setBody(body)
    # podel osy X ( je treba otocit kolem Y )
    if smer == 1:
      geom.setRotation([ 0, 0,-1, 
                         0, 1, 0, 
                         1, 0, 0 ])
    # podel osy Y ( je treba otocit kolem X )
    elif smer == 2:
      geom.setRotation([ 1, 0, 0,
                         0, 0, 1,
                         0,-1, 0 ])
    # osa Z neni treba resit  
      
    geom.shape = "cylinder" 
    geom.color = [red,green,blue]
    geom.radius = polomer
    geom.length = vyska
    geom.direction = smer

    return body, geom

def create_cappedCylinder (space,world,polomer,vyska,smer,hustota,hmotnost, red = 128, green = 128, blue = 128):
  #4-valec s kulovou cepickou
    body = ode.Body(world)
    M = ode.Mass()
    M.setCappedCylinderTotal(hmotnost,smer,polomer,vyska) # vyska nezahrnuje cepicku
    body.setMass(M)

    geom = ode.GeomCylinder(space,polomer,vyska)                  
    geom.setBody(body)
    
    # podel osy X ( je treba otocit kolem Y )
    if smer == 1:
      geom.setRotation([ 0, 0,-1, 
                         0, 1, 0, 
                         1, 0, 0 ])
    # podel osy Y ( je treba otocit kolem X )
    elif smer == 2:
      geom.setRotation([ 1, 0, 0,
                         0, 0, 1,
                         0,-1, 0 ])
    # osa Z neni treba resit  
    
    geom.shape = "cappedCylinder" 
    geom.color = [red,green,blue]
    geom.radius = polomer
    geom.length = vyska
    geom.direction = smer    

    return body, geom


