#===============================================================================
#=                            Soucast vzoroskriptu                             =
#=                                     v5                                      =
#===============================================================================
# - opraveno spatne prirazeni CFM a ERP
## 2008.03.26
# - zjednodusena zadratovana data dokud se nevyresi XODE problem s fixed jointem
 
import ode
import time
from sCreateObjects import *
from   xode        import parser
import sys
 
class Sim:
 """ trida drzici si informace o prostredi,telesech,geoms a vazbach
     - nesouvisi s vykreslovanim ani s explicitnim zadavanim pusobicich sil, momentu, ad.
 """
 #pozn: na instancni promene se odkazuji v metodach pres self, inst. prom. jsou zavadeny prirazovacim prikazem
 #      naproti tomu promene tridy jsou zavadeny deklaraci nebo prirazenim v tele tridy, v metodach se na
 #       ne nasledne odkazuje budto NazevTridy.promena, nebo self.__class__.promena
 
 world = ode.World()
 space = ode.Space()
 lasttime = 0.0     
 contactGroup = ode.JointGroup() 
 fps = 50             #vytvarim v setSimulationSpeed
 dt = 1.0/fps         #vytvarim v setSimulationSpeed - volana z konstruktoru
 listOfBodies = []    #vytvarim v loadAllFromMethod
 listOfJoints = []    #vytvarim v loadAllFromMethod
 listOfGeoms  = []    #vytvarim v loadAllFromMethod OR pri nacitani z XODE
  
 #def __init__(self, aFps=100, aErp=0.15, aCfm=1E-5, aGravity=(0,-9.80665,0) ):
 def __init__(self, aFps=100, aErp=0.8, aCfm=1E-5, aGravity=(0,-9.81,0) ):
    """ vytvori inst. prom. world,space,fps,dt """   
    
    self.setWorldParams(aErp, aCfm, aGravity)     
    self.setSimulationSpeed(aFps)
               
    return
   
 def setWorldParams(self, aErp, aCfm, aGravity=(0,-9.81,0) ):
    """nastavi v promene self.world.Gravity, ERP a CFM """
    self.world.setGravity( aGravity )  # Sets the world's global gravity vector
    self.world.setERP( aErp )          # ERP value controls how much error correction is performed in each time step, typicky od 0.1 do 0.8
    self.world.setCFM( aCfm )          # Sets CFM - constraint force mixing) value
                                       # typicky od 10E-9 do 1, vychozi je 10E-5 pro single a 10E-10 pro double precision       
    return;
    
 def setSimulationSpeed(self, aFps):   
    self.fps = aFps
    self.dt = 1.0/aFps
    return;    
    
 def cleanAll(self):
    """ vyprazdni vsechny seznamy a nastaveni simulace """
    #ToDo 
    return;    

 def makeStep(self): #_idleFunc
    """ to stejne jako _idleFunc v puvodnim navrhu - next step of simulation
        vola funkci makeNearCallback    
    """
    
    t = self.dt - (time.time() - self.lasttime)
    if(t>0):
        time.sleep(t)
        
    self.space.collide((self.world,self.contactGroup), self.makeNearCallback)  # Detect collisions and create contact joints    
    self.world.step(self.dt)           # Simulation step the world
    self.contactGroup.empty()          # Remove all contact joints
    
    self.lasttime = time.time()
    return ;

 def makeNearCallback(self,aArgs, aGeom1, aGeom2):
    """Callback function for the collide() method.
    This function checks if the given geoms do collide and
    creates contact joints if they do.
    """
   
    # Check if the objects do collide
    contacts = ode.collide(aGeom1, aGeom2)

    # Create contact joints
    self.world,self.contactgroup = aArgs
    for c in contacts:
        c.setBounce(0.8)
        c.setMu(5000)
        j = ode.ContactJoint(self.world, self.contactgroup, c)
        j.attach(aGeom1.getBody(), aGeom2.getBody())
    
    return ;

  # -- Zjisti rozmery GEOM, priradi barvu
 def __aux_DoXODEGeomWorkaround( self, root, parent ):
    geom = parent.getODEObject()
    geom.name = parent.getName()
    if 1:
      
      # Bohuzel PyODE nenacita Cylindery,
      # pouze cappedCylindery
      # Proto je treba provest premenu cappedCylinder 2 Cylinder
      if geom.name.find("@Cylinder@") > -1 and (isinstance(parent.getODEObject(), ode.GeomCapsule)):
        name      = geom.name
        space     = geom.getSpace()
        body      = geom.getBody()
                
        # Klicovy radek odstrizeni od body
        # {
        # Nalezeno na http://www.mail-archive.com/pyode-user@lists.sourceforge.net/msg00006.html 
        #geom.setBody(None)
        # }
        
        quaternion= geom.getQuaternion()
        rotation  = geom.getRotation()
        position  = geom.getPosition()    
        radius    = geom.getParams()[0]
        length    = geom.getParams()[1]
        handle    = root.namedChild(geom.name)
          
        #geom = 0
        del geom
        # Snad tohle garbage collector povoluje
        geom = ode.GeomCylinder(space, radius, length)    
        
        # Vratim telo jeho premenenemu majiteli
        geom.setBody(body)
        geom.setPosition(position)
        geom.setRotation(rotation)
        geom.setQuaternion(quaternion)
        geom.name = name
    
    if (isinstance(parent.getODEObject(), ode.GeomTriMesh)):      
      geom.NumTriangles = eval(geom.name.split(",")[4])
      
    if geom.name.count(",") >= 3: # mame tam RGB        
      geom.color = [ eval(geom.name.split(",")[1]), eval(geom.name.split(",")[2]), eval(geom.name.split(",")[3])]
    else:  
      geom.color = [128, 128, 128]
      
    # Prirazeni do shape, neni uplne nutne, pro prehlednost dobre
    if isinstance( geom, ode.GeomSphere ):
      geom.shape = "sphere"  
      geom.radius = geom.getRadius()  
      
    elif isinstance( geom, ode.GeomBox ):
      geom.shape = "box"
      geom.boxsize = (geom.getLengths()[0], geom.getLengths()[1], geom.getLengths()[2])
     
    elif isinstance( geom, ode.GeomCylinder ):
      geom.shape = "cylinder"
      geom.radius = geom.getParams()[0]   
      geom.length = geom.getParams()[1]
      
    elif isinstance( geom, ode.GeomCapsule ):
      geom.shape = "cappedCylinder"
      geom.radius = geom.getParams()[0]   
      geom.length = geom.getParams()[1]
        
    elif isinstance( geom, ode.GeomTriMesh ):
      geom.shape = "trimesh"
      
    elif isinstance( geom, ode.GeomPlane ):
      geom.shape = "plane"
      geom.a = geom.getParams()[0][0]              
      geom.b = geom.getParams()[0][1]
      geom.c = geom.getParams()[0][2]        
      geom.d = geom.getParams()[1]
      
    else:
      geom.shape = "!unknown"+str(geom)     
          
    return geom  

    
        
 def loadAllFromXode(self, aXodePath):
    """ na zaklade zadaneho XODE souboru vyplni world, space a seznamy teles listOfBodies a listOfJoints
        mela by se postarat o Gravity, ERP a CFM
    """

    if len(aXodePath) == 0:
      sys.exit()
       
    XODE_Soubor = file(aXodePath)
    XODE_Parser = parser.Parser()
    #try:
    XODE_Root   = XODE_Parser.parseFile(XODE_Soubor)
    #except:
    #  print "[!] Neplatny XODE dokument, skript bude ukoncen"      
    #  sys.exit()

    # pritomnost WORLD tagu se nemusi testovat, to je odchyceno jiz vyse
    # ( XODE_Parser.parseFile validuje soubor jako takovy )  
    self.world  = XODE_Root.namedChild('world').getODEObject()
 
    # [E] Nasledujici hodnoty jsou silne orientacni a jeste budou upraveny
    self.world.setGravity( ( 0.00, -9.80665, 0.00) )
    self.world.setERP(0.15)
    self.world.setCFM(10E-6)

    # ziskame handle na SPACE jiz automaticky vytvorene  
    self.space  = XODE_Root.namedChild('space').getODEObject()
 
    # vyprazdneni seznamu
    self.listOfGeoms  = []
    self.listOfBodies = []
    self.listOfJoints = []
    self.contactGroup = ode.JointGroup()
  
    # Ziskam seznam vsech teles
    # v prvni urovni to muzou byt jak bodies, tak geoms
    XODE_DetiSpace = XODE_Root.namedChild('space').getChildren()
    if len(XODE_DetiSpace) == 0:
      print '[!] Sekce souboru, <space>, neobsahuje zadne prvky'
      sys.exit(0)  
  
    # Projedu seznam, tag po tagu, a zjistim zda je to BODY ci GEOM
    for i in XODE_DetiSpace:    
      objekt = i.getODEObject()
      
      # Je to BODY...
      if (isinstance(objekt, ode.Body)):    
        body      = i.getODEObject()
        body.name = i.getName() 
        
        # Je to body, pripojim ho do seznamu
        self.listOfBodies.append(body)
      
        # Kdyz uz mame BODY, je cas podivat se na jeho deti,
        # Ty muzou byt bud GEOM nebo JOINT
        # se automaticky neparsuji
        XODE_DetiBodies = i.getChildren()
        
        for j in XODE_DetiBodies:
          objekt = j.getODEObject()
          
          # GEOMS
          if (isinstance(objekt, ode.GeomObject)):          
            geom = self.__aux_DoXODEGeomWorkaround(XODE_Root, j)
            self.listOfGeoms.append(geom)
          
          # JOINTS  
          elif (isinstance(objekt, ode.Joint)):
            joint = j.getODEObject()
            print j.getName()
            joint.name = j.getName()
            
            # [E] ZDE se musi odchytit "vadne" jointy a pretransformovat            
            self.listOfJoints.append(joint)
             
      # Je to GEOM...
      elif (isinstance(objekt, ode.GeomObject)):    
        geom = self.__aux_DoXODEGeomWorkaround(XODE_Root, i)
        self.listOfGeoms.append(geom)

    print "pocet teles v simulaci (bez podlahy):", len(self.listOfBodies)
    print "pocet vazeb v simulaci:", len(self.listOfJoints)
    
    return ;

 def getBodyByName( self, bName ):
   for b in self.listOfBodies:
     if b.name == bName:
       return b
   return None
   
 def getJointByName( self, jName ):
   for j in self.listOfJoints:
     if j.name == jName:
       return j
   return None   
 
 def loadAllFromMethod(self):
    """- dokud nebude hotova prace na nacitani z xode souboru, bude se volat tato funkce v niz budou "na tvrdo" zadany telesa a vazby
       - metoda vyplni world, space a seznamy teles listOfBodies a listOfJoints
       - bude pouzivat 1. skript sCreateObjects.py pro vytvareni teles
                       2. setWorldParams()
    """                    
    self.listOfBodies = []
    self.listOfGeoms  = []
    self.listOfJoints = []

         
################################################################################
##telesa
    #0 koule_1
    body, geom = create_sphere(self.space,self.world,0.2,1.0)   #(space,world,polomer,hustota,hmotnost, red = 128, green = 128, blue = 128)
    body.setPosition( (-1,2.0,0) )                     # nastaveni polohy
    #body.addForce( (0,1,0) )                           # nastaveni vychozi pusobici sily       
    self.listOfBodies.append(body)                          # nove vytvorene body pridam do seznamu
    self.listOfGeoms.append(geom)
    #4 kvadr_2
    body, geom = create_box(self.space, self.world, 0.1, 1.0, 0.3, 1.0, 255, 255, 100)   #(space,world,hranaX,hranaY,hranaZ,hustota,hmotnost, red = 128, green = 128, blue = 128)
    body.setPosition( (-1,1.0,0) ) 
    #body.addForce( (250,0,0) )
    self.listOfBodies.append(body)
    self.listOfGeoms.append(geom)           
    
##vazby    
  #0 FixedJoint -vytvoreni pevne vazby mezi koule_1 a prostredim  
    #vazba = ode.FixedJoint(self.world) # vytvorim pevne spojeni dvou teles
    #vazba.attach(self.listOfBodies[0],ode.environment) # attach(body1, body2)
    #vazba.setFixed()    #zamknuti pro zajisteni vzajemneho relativniho posunuti a natoceni
                        #  call this on the fixed joint after it has been attached to remember the current
                        #  desired relative offset and desired relative rotation between the bodies
    #self.listOfJoints.append(vazba) #pridam do seznamu vazeb
    
  #2 Hinge2Joint -vytvoreni sfericke vazby, ktera ma 2 mimobezne osy, mezi koule_1 a kvadr_2
    vazba = ode.Hinge2Joint(self.world) # vytvorim pevne spojeni dvou teles
    vazba.attach(self.listOfBodies[0],self.listOfBodies[1]) # attach(body1, body2)
                                                  # urcim pro ktera telesa se ma vzba pouzit
    vazba.setAnchor((-1.0,2,0.0))   # set the hinge anchor which must be given in world coordinates
    vazba.setAxis1((0,1,0))         # set the hinge axis1
    vazba.setAxis2((0,0,1))         # set the hinge axis2                                                     
    self.listOfJoints.append(vazba) # pridam do seznamu vazeb

    print "pocet teles v simulaci (bez podlahy):", len(self.listOfBodies)
    print "pocet vazeb v simulaci:", len(self.listOfJoints)
################################################################################

    #vyplnim seznamy teles a vazeb
    #co udelam s podlahou...resp. co udelam s cimkoliv podobnym ? !!! 
    
         
    return ;    
 