#===============================================================================
#=                            Soucast vzoroskriptu                             =
#=                                     v5                                      =
#===============================================================================
from sSim import Sim
from sPaintSim import PaintSim
import math 

class TaskToSim:
  """ zajisteni vseho co souvisi s konkretnim ukolem
     pust simulaci s temito parametry, (ne)pust vykreslovani tehdy a tehdy s takovymi parametry
  """
  sim   = Sim()
  paint = PaintSim(640, 480, "Simulace robotu Pierot")
  
  def __init__(self):
    self.TotalTime = 0.0
    self.MaxTime   = 0.0  
    
    self.a, self.v = 0, 0
    self.Kp = 0.4
    self.Ki = 0.00043
    self.Kd = 2.5
    self.angleold = 0
    self.isum = 0       
    return;
    
  def matrix2axis(self, m):
    """Convert from PyODE rotation matrix to axis angles

    Original (in Java) from:
    http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToAngle/
    """
    (
            M00, M01, M02,
            M10, M11, M12,
            M20, M21, M22,
    ) = (
            0, 3, 6,
            1, 4, 7,
            2, 5, 8,
    )
    epsilon = 0.01
    if math.fabs(m[M01] - m[M10]) < epsilon and math.fabs(m[M02] - m[M20]) < epsilon and math.fabs(m[M12] - m[M21]) < epsilon:
      if math.fabs(m[M01] + m[M10]) < 0.1 and math.fabs(m[M02] + m[M20]) < 0.1 and math.fabs(m[M12] + m[M21]) < 0.1 and math.fabs(m[M00] + m[M11] + m[M22]) - 3 < 0.1:
        return (0, 1, 0, 0)
      angle = math.pi
      x = (m[M00] + 1) / 2
      if x > 0:
              x = math.sqrt(x)
      else:
              x = 0
      y = (m[M11] + 1) / 2
      if y > 0:
              y = math.sqrt(y)
      else:
              y = 0
      z = (m[M22] + 1) / 2
      if z > 0:
              z = math.sqrt(z)
      else:
              z = 0
      xzero = math.fabs(x) < epsilon
      yzero = math.fabs(y) < epsilon
      zzero = math.fabs(z) < epsilon
      xypos = m[M01] > 0
      xzpos = m[M02] > 0
      yzpos = m[M12] > 0
      if xzero and not yzero and not zzero:
              if not yzpos:
                      y = -y
      elif yzero and not zzero:
              if not xzpos:
                      z = -z
      elif zzero:
              if not xypos:
                      x = -x
      return (angle, x, y, z)
      
    s = math.sqrt((m[M21] - m[M12]) ** 2 + (m[M02] - m[M20]) ** 2 + (m[M10] - m[M01]) ** 2)
    
    if math.fabs(s) < 0.001:
      s = 1
      
    angle = math.acos((m[M00] + m[M11] + m[M22] - 1) / 2)
    x = (m[M21] - m[M12]) / s
    y = (m[M02] - m[M20]) / s
    z = (m[M10] - m[M01]) / s
    
    return (angle, x, y, z)    
 
  def setTaskParams(self):
    """ spousti se ve stejny okamzik jako idleFunc
        nastavuje sily a momenty dle potreby,
        vypocitava akcni zasahy apod.          
    """   
      
    return
    
  def runTaskTime(self,aDuration):
    self.TotalTime = 0.0
    while self.TotalTime < aDuration:
        self.sim.makeStep()
        self.TotalTime += self.sim.dt
        print "time", self.TotalTime
        
    self.postprocessTask()                      
    return

  # Uzitecne hlavne pro GLUT    
  def runTaskTime_Single(self):
   
    ###
    bTelo = self.sim.getBodyByName("Pierot_Telo")
    bKolo1= self.sim.getBodyByName("Pierot_Kolo1")
    joint1= self.sim.getJointByName("joint1")
    joint2= self.sim.getJointByName("joint2")
    
    r = bTelo.getRotation()

    angle, x, y, z = self.matrix2axis(r)
    angle = angle * 180 * z / math.pi
    
    # Rozviklani pierota
    if (self.TotalTime > 0.1) and (self.TotalTime < 0.11):
    
        joint1.addTorque(500)
        joint2.addTorque(500)
        
    else:
    
      self.a = self.Kp * angle + self.Ki * self.isum + self.Kd * (angle-self.angleold)                  
      self.v = self.a
      joint1.addTorque(self.v)
      joint2.addTorque(self.v) 
  
      self.isum += angle
      self.angleold = angle
      
      posx, posy, posz = bKolo1.getPosition()
          
      print "%1.2fs: uhel=(%6.3f) moment=(%6.3f) x=(%6.3f))" % \
           (self.TotalTime, angle, self.v, posx)

    self.sim.makeStep()
    self.TotalTime += self.sim.dt

    if self.TotalTime > self.MaxTime:
      return -1
    else:
      return 0                
 
  # Uzitecne hlavne pro GLUT   
  def postprocessTask(self):
    """ nez GLUT odstreli aplikaci, milosrdne zavola tuto funkci,
        takze TADY a jedine tady lze napr. ulozit vysledky atd.
    """
    print("-- Vysledek simulace")
    bKolo1= self.sim.getBodyByName("Pierot_Kolo1")
    print "Kolo1 je na pozici: ", bKolo1.getPosition()
    
    return
    
  def runTaskTime_Graphical(self, MaxTime):
    self.TotalTime = 0
    self.MaxTime   = MaxTime
    self.paint.Visualize( self.sim.listOfGeoms, self.runTaskTime_Single, self.postprocessTask )  
