#===============================================================================
#=                          Zobrazovatko XODE souboru                          =
#=                            Petr Schreiber, 2008                             =
#=                                    v10                                      =
#=                                    SP2                                      =
#===============================================================================

# -- Tento program nepouziva unifikovanou strukturu programu, slouzi jen a pouze
# -- pro rychle zobrazeni obsahu XODE souboru 

# -- Nasledujici radek zaruci korektni deleni pro floating point cisla 
from __future__ import division

# -- Matematicky modul, pouzit pro funkci odmocniny
import math                      

# -- Casomerny modul, a obecne pouzitelne moduly pro praci s retezci a soubory
import time, string, os, sys

# -- Moduly pro 3D grafiku
from   OpenGL.GL   import *
from   OpenGL.GLU  import *
from   OpenGL.GLUT import *

# -- Moduly pro fyzikalni vypocty
from   xode        import parser
import ode

# -- Dialog vyberu souboru
from tkFileDialog   import askopenfilename

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# ODE 
#-------------------------------------------------------------------------------
def _NearCallback(args, geom1, geom2):
  """ Tato funkce je volana, pro zpracovani kolizi 2 geoms"""
  """ Vytvari docasne joints """

  # -- Zpristupni globalni promenne
  global world
  global contactgroup
  
  # -- Seznam kolizi
  contacts = ode.collide(geom1, geom2)
    
  # -- Vytvori klouby
  world, contactgroup = args
  for c in contacts:
    c.setBounce(0.2)
    c.setMu(0.1)
    j = ode.ContactJoint(world, contactgroup, c)
    j.attach(geom1.getBody(), geom2.getBody())

#-------------------------------------------------------------------------------
# OpenGL funkce
#-------------------------------------------------------------------------------
def _CreateWindow(windowTitle, width, height,):
    glutInit ([])                                     # Inicializace GLUT
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGBA )   # Double buffer redukuje blikani pri vykreslovani, RGBA mod je nejrychlejsi  

    glutInitWindowPosition (-1, -1)                   # system si umisti okno sam
    glutInitWindowSize (width, height)                # nastaveni velikosti okna
    glutCreateWindow (windowTitle)                    # vytvoreni okna

    # -- Nasleduje nastaveni OpenGL
    # -- OpenGL pracuje jako "state machine" - tzn. pokud neco zapnu tak to zustane zapnute dokud to nevypnu :)

    # -- Nastaveni svetla
    glLightfv(GL_LIGHT0,GL_POSITION,[15,15,15,0])     # Umisti svetlo na pozici 15, 15, 15
    glLightfv(GL_LIGHT0,GL_DIFFUSE ,[1,1,1,1])        # Svetlo se jevi jako bile ( RGB se slozkami v intervalu [0,1] )
    glLightfv(GL_LIGHT0,GL_SPECULAR,[1,1,1,1])        # Bile odrazy
    glEnable (GL_LIGHT0)                              # Zapne jeden svetelny zdroj
    glEnable(GL_LIGHTING)                             # Zapne nasvetleni jako takove

    glEnable(GL_COLOR_MATERIAL)                       # Umoznuje pusobeni svetla pri zachovani barvy telesa
    glShadeModel(GL_SMOOTH)                           # Relativne plynuly prechod svetla ( per vertex )
    glEnable(GL_NORMALIZE)                            # Automaticky normalizuje normaly (-Schr.)
    glEnable (GL_AUTO_NORMAL)                         # Automaticky normalizuje normaly i po zmene rozmeru pomoci glScale<pismeno> (-Schr.) 
    
    glEnable(GL_DEPTH_TEST)                           # Zajisti spravne vykreslovani ( poradi ) fragmentu primitiv  
    glDepthMask(GL_TRUE)                              
    glDepthFunc(GL_LESS)                              # Zapisuje pouze fragmenty s mensi hloubkou nez je stavajici                            


    glViewport(0,0,width,height)                      # Nastavi "okno v okne" do ktereho se bude kreslit,
                                                      # musi byt mensi nebo stejne velke jako klientska oblast okna
    
    # -- nastaveni transformacnich matic           			
    glMatrixMode(GL_PROJECTION) # Nastaveni projekce
    glLoadIdentity()            # Nacte do GL_PROJECTION matici [ 1, 0, 0 ]
                                #                               [ 0, 1, 0 ]   
                                #                               [ 0, 0, 1 ]
    # -- Perspektiva, 45 deg FOV ( Field-Of-View ), spravny pomer stran a predni a zadni orezovou rovinu                                                                                            
    gluPerspective (45, width/height, 0.1, 512)

    glMatrixMode(GL_MODELVIEW)  # Matice modelview se stara o posouvani objektu ve scene
    glLoadIdentity()            # Nacte do GL_MODELVIEW matici  [ 1, 0, 0 ]
                                #                               [ 0, 1, 0 ]   
                                #                               [ 0, 0, 1 ]

    glClearColor(0.0,0.25,0.5,1.0)                    # vychozi barva pozadi
    glColor3ubv ([255, 255, 255])                     # vychozi barva teles
    
    
def OpenGL_VykresliValec( radius, vyska, kryty ):

  pi    = 3.14159265358
  twopi = 6.28318530716
  halfpi= 1.57079632679
  height = vyska  
  
  # slozitost
  n = 15  
  j = ((n/2)-2)/2
  
  theta1 = j*twopi/n - halfpi
  theta2 = (j+1)*twopi/n - halfpi
  glBegin (GL_QUAD_STRIP)
  for i in range(0, n+1):
    theta3 = i*twopi/n
    x  = math.cos(theta2) * math.cos(theta3) 
    y  = -math.cos(theta2) * math.sin(theta3)
    z  = 0 

    px = radius*x
    py = radius*y
    glNormal3f (x, y, z)
    glVertex3f (px, py,-vyska/2)#,pz)

    x  = math.cos(theta2) * math.cos(theta3) 
    y  = -math.cos(theta2) * math.sin(theta3)
    z  = 0
    px = radius*x
    py = radius*y
    glNormal3f (x, y, z)
    glVertex3f (px, py, vyska/2)

  glEnd() #GL_QUAD_STRIP

  if kryty == 0:
    theta1 = j*twopi/n - halfpi
    theta2 = (j+1)*twopi/n - halfpi
    glBegin (GL_TRIANGLE_FAN)
    glNormal3f(0.0, 0.0, -1.0)
    glVertex3f(0.0,0.0,-vyska/2)
    for i in range(0, n+1):
      theta3 = i*twopi/n

      x  = math.cos(theta2) * math.cos(theta3)
      y  = math.cos(theta2) * math.sin(theta3)
      
      px = radius*x
      py = radius*y

      glVertex3f (px,-py,-vyska/2)

    glEnd() #GL_TRIANGLE_FAN

    theta1 = j*twopi/n - halfpi
    theta2 = (j+1)*twopi/n - halfpi
    glBegin (GL_TRIANGLE_FAN)
    glNormal3f (0.0, 0.0, 1.0)
    glVertex3f (0.0,0.0,vyska/2)
    for i in range(0,n+1):
      theta3 = i*twopi/n

      x  = math.cos(theta2) * math.cos(theta3)
      y  = math.cos(theta2) * math.sin(theta3)
      px = radius*x
      py = radius*y
      glVertex3f (px,py,vyska/2)
    glEnd() #GL_TRIANGLE_FAN
  else:
    glTranslatef (0.0,0.0,vyska/2)  # pro geomCapsule ( = cappedCylinder )
    glutSolidSphere(radius,15,15)
    glTranslatef (0.0,0.0,-vyska)
    glutSolidSphere(radius,15,15)
       
def OpenGL_PrevedXODEGeomNaDListy():
  global XODE_ListGeoms
  
  DListID = 1

  if len(XODE_ListGeoms):
    
    # Display listy - ultimatni optimalizace do kazde domacnosti
    # Vygeneruje presne tolik DListu, kolik bude objektu 
    glGenLists(len(XODE_ListGeoms))
    
    for teleso in XODE_ListGeoms:
      # Kompilace listu
      glNewList (DListID, GL_COMPILE)
      glPushMatrix
      # Pokud je teleso pruhledne, pozadavek na zpruhledneni se ulozi do listu
      # Pro odliseni pruhlednych teles na obrazovce pouzivam ruznou alpha
      if teleso.name.find("--transparent") > -1:
        glBlendFunc(GL_SRC_COLOR, GL_ONE)
        glDepthMask(GL_FALSE)
        glEnable(GL_BLEND)
        glColor4ub(teleso.r, teleso.g, teleso.b, 128)
      else:        
        glColor4ub(teleso.r, teleso.g, teleso.b, 255)
      
      # Dle typu objektu vznikne korektni reprezentace        
      if (isinstance(teleso, ode.GeomBox)):
        glScalef(teleso.getLengths()[0], teleso.getLengths()[1], teleso.getLengths()[2])                 
        glutSolidCube(1)        
      elif (isinstance(teleso, ode.GeomSphere)):                
        glutSolidSphere(teleso.getRadius(), 12, 12)
      elif (isinstance(teleso, ode.GeomCylinder)):
        OpenGL_VykresliValec(teleso.getParams()[0], teleso.getParams()[1], 0)                
      elif (isinstance(teleso, ode.GeomCapsule)):
        OpenGL_VykresliValec(teleso.getParams()[0], teleso.getParams()[1], 1)                                 
      elif (isinstance(teleso, ode.GeomPlane)):           # pozor, funguje jen pro plochy 0,1,0 ZATIM
        a = teleso.getParams()[0][0]              
        b = teleso.getParams()[0][1]
        c = teleso.getParams()[0][2]        
        d = teleso.getParams()[1]
        dx = math.degrees(math.atan2(c, b))
        dz = -math.degrees(math.atan2(a, b))
        #print dx, dz
        glTranslatef(0,d,0)
        glRotatef (dz,0,0,1)
        glRotatef (dx,1,0,0)    
                                       
        glBegin(GL_QUADS)        
        glNormal3f(0, 1, 0)                
        glVertex3f(-100, 0,  100)
        glVertex3f( 100, 0,  100)
        glVertex3f( 100, 0, -100)
        glVertex3f(-100, 0, -100)         
        glEnd()         
        glPointSize(2)
        glBegin(GL_POINTS)
        glColor3ub(0,0,0)
        for i in range(-100, 100):
          for j in range(-100, 100):
            glVertex3f(i, 0, j)
        glEnd()
        glPointSize(1)
      elif (isinstance(teleso, ode.GeomTriMesh)):        
        glBegin(GL_TRIANGLES)
        x, y, z = teleso.getPosition()
        ux, uy, uz = 0, 0, 0
        vx, vy, vz = 0, 0, 0    
        
        for t in range(0, teleso.NumTriangles):
          # Vypocet normaloveho vektoru
          ux = teleso.getTriangle(t)[1][0] - teleso.getTriangle(t)[0][0]  
          uy = teleso.getTriangle(t)[1][1] - teleso.getTriangle(t)[0][1]
          uz = teleso.getTriangle(t)[1][2] - teleso.getTriangle(t)[0][2]
          
          vx = teleso.getTriangle(t)[2][0] - teleso.getTriangle(t)[0][0]  
          vy = teleso.getTriangle(t)[2][1] - teleso.getTriangle(t)[0][1]
          vz = teleso.getTriangle(t)[2][2] - teleso.getTriangle(t)[0][2]
          
          nx = (uy * vz - uz * vy)
          ny = -(ux * vz - uz * vx)
          nz = (ux * vy - uy * vx)
          
          nl = math.sqrt(nx*nx + ny*ny + nz*nz)
    
          nx = nx / nl
          ny = ny / nl
          nz = nz / nl
          
          # Normalovy vektor staci zadat pro prvni vertex v trojuhelniku
          glNormal3f(nx, ny, nz)           
          
          if teleso.getBody() != None:
            for v in range(0,3):
              glVertex3f(teleso.getTriangle(t)[v][0]-x, teleso.getTriangle(t)[v][1]-y, teleso.getTriangle(t)[v][2]-z)
          else:
            for v in range(0,3):
              glVertex3f(teleso.getTriangle(t)[v][0], teleso.getTriangle(t)[v][1], teleso.getTriangle(t)[v][2])
              
            
        glEnd()

      # Pripadnou pruhlednost je treba pred ukoncenim listu vypnout
      if teleso.name.find("--transparent") > 0:
        glDisable(GL_BLEND)
        glDepthMask(GL_TRUE)
                                 
      glPopMatrix  
      glEndList()
      
      teleso.DList = DListID
      DListID += 1  
                              
def OpenGL_UvolniDisplayListy():
  global XODE_ListGeoms
  glDeleteLists(0, len(XODE_ListGeoms))     

def OpenGL_VykresliTeleso( teleso ):

  if ((isinstance(teleso, ode.GeomPlane))) or ((isinstance(teleso, ode.GeomTriMesh)) and teleso.getBody() == None):
      rot = [1.0, 0.0, 0.0, 0.0,
             0.0, 1.0, 0.0, 0.0,
             0.0, 0.0, 1.0, 0.0,
             0.0, 0.0, 0.0, 1.0]            
  else:
    x,y,z = teleso.getPosition()
    R = teleso.getRotation()
    rot = [R[0], R[3], R[6], 0.0,
           R[1], R[4], R[7], 0.0,
           R[2], R[5], R[8], 0.0,
           x, y, z, 1.0]
    
  glPushMatrix()       
  glMultMatrixd(rot)
  glCallList (teleso.DList)       # telesa jsou jiz v display listech
  glPopMatrix()
       

def OpenGL_VykresliVazby():
  global XODE_ListJoints 
  glPushMatrix()
  if len(XODE_ListJoints) > 0:
    glDisable(GL_LIGHTING)
    glBegin(GL_LINES)             # dost dulezite dat to mimo smycku, grafarna jinak dostane skytavku
    glColor3ub(255, 255, 0)
          
    for joint in XODE_ListJoints:        
      x1, y1, z1 = joint.getBody(0).getPosition()        
      
      if (isinstance(joint.getBody(1), ode.Body)):
        x2, y2, z2 = joint.getBody(1).getPosition()
      else:
        try:  
          x2, y2, z2 = joint.getAnchor()
          
        except:
          x2, y2, z2 = x1, y1, z1
        
      glVertex3f(x1, y1, z1)  
      glVertex3f(x2, y2, z2)
               
    glEnd()
    glEnable(GL_LIGHTING)     

  glPopMatrix()
  
  
def _DrawText( TextToPrint, x, y ):
  # Vykresli text, jeho pozice je delana jako v DOSovskych BASICich,
  # tzn. 1,1 je horni levy roh
  x = (x-1)*18
  y = (y-1)*18
  
  # Svetlo neni treba, naopak dokonce vadi :)
  glDisable(GL_LIGHTING)
  
  glMatrixMode(GL_PROJECTION)
  matrix   = glGetDouble( GL_PROJECTION_MATRIX )
  # Z tohoto ziskam rozmery okna
  viewport = glGetInteger( GL_VIEWPORT )
  
  glLoadIdentity()
  gluOrtho2D(0.0, viewport[2], 0.0, viewport[3])
  glMatrixMode(GL_MODELVIEW)
  glPushMatrix()
  glLoadIdentity()
  glRasterPos2i(x, viewport[3]-y-18)
  lines = 0
  
  for Letter in TextToPrint:
  		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, ord(Letter))
  		
  glPopMatrix()
  
  # Uklid - uvadeni do puvodniho stavu:
  glMatrixMode(GL_PROJECTION)
  glLoadMatrixd( matrix ) # should have un-decorated alias for this...
  glMatrixMode(GL_MODELVIEW)
  glEnable(GL_LIGHTING)
      
def _DrawFunc():
  global bodies, SnimekHotov, CasSnimku, KameraMod
  global KameraRotBodX, KameraRotBodY, KameraRotBodZ
  
  SnimekHotov = 0
  cas1 = time.clock()    
  
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )  # Smaze buffery ( obraz + hloubka )
    
  glLoadIdentity()                                      # Resetuje transformace
  # -> Graficky kod
     
  
  if KameraMod == 0:
    glTranslatef (0.0, -KameraVyska, -KameraVzdalenost)
    glRotatef (KameraUhelY, 0, 1, 0)
    glTranslatef (-KameraRotBodX, -KameraRotBodY, -KameraRotBodZ)
   
  elif KameraMod == 1:
    x,y,z = XODE_ListBodies[KameraFokus].getPosition()      
    glTranslatef (0.0, -KameraVyska, -KameraVzdalenost)
    glRotatef (KameraUhelY, 0, 1, 0)
    glTranslatef (-x, -y, -z)  

  # Vykresleni vsech teles
  
  # -- Predni strany zobrazeny "jako normalne"
  glPolygonMode(GL_FRONT, GL_FILL)
  # -- Predni strany zobrazeny jako "drateny model"
  glLineWidth(3)  
  glPolygonMode(GL_BACK, GL_LINE)  

  if len(XODE_ListGeoms) > 0:
    # Pevna telesa
    for g in XODE_ListGeoms:
      OpenGL_VykresliTeleso(g)   


  # Jointy
  OpenGL_VykresliVazby()
  
  # -< Graficky kod
  glLoadIdentity()                                      # Resetuje transformace  
  glDisable(GL_DEPTH_TEST)  
  glColor3ub(255, 255, 255)    
  if KameraMod == 1:
    _DrawText("Sledujete objekt \""+XODE_ListBodies[KameraFokus].name+"\"", 1, 1)
    _DrawText("X: %.3f" % (XODE_ListBodies[KameraFokus].getPosition()[0]), 1, 2)
    _DrawText("Y: %.3f" % (XODE_ListBodies[KameraFokus].getPosition()[1]), 1, 3)    
    _DrawText("Z: %.3f" % (XODE_ListBodies[KameraFokus].getPosition()[2]), 1, 4)
  else:
    if len(XODE_ListBodies) != 0: 
      _DrawText("Stisknete prave tlacitko pro vyber objektu ke sledovani", 1, 1)
      
  glEnable(GL_DEPTH_TEST)

  #ZjistiPripadnyKlik()    
  glutSwapBuffers()
  
  SnimekHotov = 1
  cas2 = time.clock()
  
  CasSnimku = (cas2-cas1)
  if CasSnimku < 0.001:
    CasSnimku = 0.001
        
#-------------------------------------------------------------------------------
# GLUT callbacky
#------------------------------------------------------------------------------- 

def _IdleFunc():
  global world, SnimekHotov, CasSnimku, SimStep, space, SimulaceZastavena
  
  if SimulaceZastavena != 0.0:
    # Detect collisions and create contact joints
    space.collide((world, XODE_ContactGroup), _NearCallback)
  
    # Simulation step
    #n = int(CasSnimku/SimStep+0.6)
    #print n
    #print CasSnimku, n
    #for i in xrange(n):
    #  world.step(SimStep/n)
    
    for i in range(0, 10):    
      world.step(SimStep)
  
    # Remove all contact joints
    XODE_ContactGroup.empty()
  
  if SnimekHotov == 1:    
    # print "Idle -> prosim scenu o vykresleni"    
    _DrawFunc()
    # print "Idle <- uz se vykreslila"  

def GLUT_CBKlavesnice(pismeno, x, y):
  "Zvlada jen pismena, jine klavesy jsou problem"
  global SimStep
  if pismeno == 'q':
    sys.exit (0)  
    
  if pismeno == 'a':
    XODE_ListBodies[KameraFokus].addForce([-10000,0,0])
    
  if pismeno == 'd':
    XODE_ListBodies[KameraFokus].addForce([10000,0,0])  

  if pismeno == 'w':
    XODE_ListBodies[KameraFokus].addForce([0,0,-10000])
    
  if pismeno == 's':
    XODE_ListBodies[KameraFokus].addForce([0,0,10000])  
      
  if pismeno == 'f':
    print "FPS:", 1/SimStep  
    
    
    
def GLUT_CBKlavesnicePlus(klavesa, x, y):
  "Zvlada jen specialni klavesy"
  global KameraVzdalenost, KameraVyska, KameraUhelY, CasSnimku

  if klavesa == GLUT_KEY_DOWN:
    KameraVzdalenost += 1         

  if klavesa == GLUT_KEY_UP:
    KameraVzdalenost -= 1
    if KameraVzdalenost < -1.0:
      KameraVzdalenost = -1.0            

  if klavesa == GLUT_KEY_RIGHT:    
    KameraUhelY -= 1    
        
  if klavesa == GLUT_KEY_LEFT:
    KameraUhelY += 1
        

  if klavesa == GLUT_KEY_PAGE_UP:
    KameraVyska += 1
        

  if klavesa == GLUT_KEY_PAGE_DOWN:    
    KameraVyska -= 1
    if KameraVyska < 1.0:
      KameraVyska = 1.0    
      
def GLUT_CBVykresli():
  OpravduRadsiNic = 1      

def GLUT_CBZmenaRozmeru( x, y ):
  # Viewport se roztahne pres celou klientskou oblast
  glViewport (0, 0, x, y)
  
  # Nastavi se znova FOV, pomer stran a orezavaci roviny
  glMatrixMode (GL_PROJECTION)
  glLoadIdentity()
  gluPerspective (45, x/y, 0.1, 512)
  
  # 
  glMatrixMode (GL_MODELVIEW)    
  glLoadIdentity()

#-------------------------------------------------------------------------------
# Funkce pro cteni XODE
#-------------------------------------------------------------------------------

# -- Zjisti rozmery GEOM, priradi barvu
def XODE_ZpracujGeom( Rodic ):
  global XODE_Root
  
  geom = Rodic.getODEObject()
  geom.name = Rodic.getName()
  
  # Bohuzel PyODE nenacita Cylindery,
  # pouze cappedCylindery
  # Proto je treba provest premenu cappedCylinder 2 Cylinder
  if geom.name.find("@Cylinder@") > -1 and (isinstance(Rodic.getODEObject(), ode.GeomCapsule)):
    name      = geom.name
    space     = geom.getSpace()
    body      = geom.getBody()
    
    # Klicovy radek odstrizeni od body
    # {
    # Nalezeno na http://www.mail-archive.com/pyode-user@lists.sourceforge.net/msg00006.html 
    geom.setBody(None)
    # }
    
    quaternion= geom.getQuaternion()
    rotation  = geom.getRotation()
    position  = geom.getPosition()    
    radius    = geom.getParams()[0]
    length    = geom.getParams()[1]
    handle = XODE_Root.namedChild(geom.name)
      
    geom = 0
    geom = ode.GeomCylinder(space, radius, length)    
    
    geom.setBody(body)
    geom.setPosition(position)
    geom.setRotation(rotation)
    geom.setQuaternion(quaternion)
    geom.name = name
  
  if (isinstance(Rodic.getODEObject(), ode.GeomTriMesh)):      
    geom.NumTriangles = eval(geom.name.split(",")[4])
    
  if geom.name.count(",") >= 3: # mame tam RGB        
    geom.r = eval(geom.name.split(",")[1])
    geom.g = eval(geom.name.split(",")[2])
    geom.b = eval(geom.name.split(",")[3])          
  else:  
    geom.r = 128
    geom.g = 128
    geom.b = 128

  return geom            

# -- Nacte XODE soubor
def XODE_NactiSoubor( souborJmeno ):
  global space, world
  global XODE_ListBodies, XODE_ListGeoms, XODE_ListJoints, XODE_ContactGroup, XODE_Root
  
  if len(souborJmeno) == 0:
    sys.exit() 

     
  XODE_Soubor = file(souborJmeno)
  XODE_Parser = parser.Parser()
  try:
    XODE_Root   = XODE_Parser.parseFile(XODE_Soubor)
  except:
    print "[!] Neplatny XODE dokument, skript bude ukoncen"      
    sys.exit()

  # pritomnost WORLD tagu se nemusi testovat, to je odchyceno jiz vyse
  # ( XODE_Parser.parseFile validuje soubor jako takovy )  
  world  = XODE_Root.namedChild('world').getODEObject()

  # [E] Nasledujici hodnoty jsou silne orientacni a jeste budou upraveny
  world.setGravity( ( 0.00, -9.80665, 0.00) )
  world.setERP(0.2)
  world.setCFM(10E-5)

  # ziskame handle na SPACE jiz automaticky vytvorene  
  space  = XODE_Root.namedChild('space').getODEObject()

  # vyprazdneni seznamu
  XODE_ListGeoms  = []
  XODE_ListBodies = []
  XODE_ListJoints = []
  XODE_ContactGroup = ode.JointGroup()

  # Ziskam seznam vsech teles
  # v prvni urovni to muzou byt jak bodies, tak geoms
  XODE_DetiSpace = XODE_Root.namedChild('space').getChildren()
  if len(XODE_DetiSpace) == 0:
    print '[!] Sekce souboru, <space>, neobsahuje zadne prvky'
    sys.exit()  

  # Projedu seznam, tag po tagu, a zjistim zda je to BODY ci GEOM
  for i in XODE_DetiSpace:    
    objekt = i.getODEObject()
    
    # Je to BODY...
    if (isinstance(objekt, ode.Body)):    
      body      = i.getODEObject()
      body.name = i.getName() 
      
      # Je to body, pripojim ho do seznamu
      XODE_ListBodies.append(body)
    
      # Kdyz uz mame BODY, je cas podivat se na jeho deti,
      # Ty muzou byt bud GEOM nebo JOINT
      # se automaticky neparsuji
      XODE_DetiBodies = i.getChildren()
      
      for j in XODE_DetiBodies:
        objekt = j.getODEObject()
        
        # GEOMS
        if (isinstance(objekt, ode.GeomObject)):          
          geom = XODE_ZpracujGeom(j)
          XODE_ListGeoms.append(geom)
        
        # JOINTS  
        elif (isinstance(objekt, ode.Joint)):
          joint = j.getODEObject()
          # [E] ZDE se musi odchytit "vadne" jointy a pretransformovat            
          XODE_ListJoints.append(joint)
           
    # Je to GEOM...
    elif (isinstance(objekt, ode.GeomObject)):    
      geom = XODE_ZpracujGeom(i)
      XODE_ListGeoms.append(geom)

# -- Experimentalni zjisteni mista kliku
def ZjistiPripadnyKlik():
  "... nespolehlive :("
  global KameraRotBodX, KameraRotBodY, KameraRotBodZ, KameraVyska, KameraVzdalenost, MyskaX, MyskaY
  if MyskaX != 0:
    
    # Provede vsechny graficke operace
    glFlush()
    glFinish()
    
    # Ziskam OpenGL matice
    viewport   = glGetIntegerv(GL_VIEWPORT)
    mvmatrix   = glGetDoublev (GL_MODELVIEW_MATRIX)
    projmatrix = glGetDoublev (GL_PROJECTION_MATRIX)
    # Pozice kurzoru
    winx = MyskaX    
    winy = MyskaY
    
    print 
    # Z-Buffer - hodnota v rozmezi [0,1]
    winz = glReadPixelsf( winx, winy, 1, 1, GL_DEPTH_COMPONENT)    
    dx, dy, dz = gluUnProject (winx, viewport[3]-winy-1, winz[0][0], mvmatrix, projmatrix, viewport )
    
    KameraRotBodX    = dx 
    KameraRotBodY    = dy
    KameraRotBodZ    = dz 
    
    print "Kliknuti na", dx, dy, dz
    MyskaX = 0
    MyskaY = 0 

def GLUT_CBMys(button, state, x, y):
  global MyskaX, MyskaY
  if button == GLUT_LEFT_BUTTON and state == GLUT_DOWN:
    MyskaX = x
    MyskaY = y
    
  
 

#-------------------------------------------------------------------------------
# Menu callbacky
#-------------------------------------------------------------------------------   
      
def kontextoveMenu_KameraFokus( id ):
  global KameraFokus, KameraMod, KameraVyska
  KameraFokus = id
  KameraMod = 1 # Body kamera
  KameraVyska = 0
  
def kontextoveMenu_Hlavni( id ):
  global KameraMod, SimulaceZastavena
  if id == 1000:
    sys.exit (0)
  elif id == 998:
    SimulaceZastavena = not SimulaceZastavena   
  elif id == 999:
    KameraMod = 0 # Globalni pohled
      
#-------------------------------------------------------------------------------
# Zacatek samotneho programu
#-------------------------------------------------------------------------------   
_CreateWindow("XODE Viewer, v10; Stisknete prave tlacitko mysi pro kontextovou nabidku", 640, 480)
XODE_Root = 0
XODE_NactiSoubor( askopenfilename(title='Vyberte XODE scenu', filetypes=[('XODE soubor', '*.xode')]))#'scena_mistnost.xode' )

time.clock()
OpenGL_PrevedXODEGeomNaDListy()
SimStep  = 0.001                             # Krok simulace 

KameraRotBodX    = 0.0
KameraRotBodY    = 0.0
KameraRotBodZ    = 0.0

KameraVzdalenost = 10.0                  # Pocatecni pozice kamery
KameraVyska      = 5.0
KameraUhelY      = 45.0
KameraFokus      = 0
KameraMod        = 0
MyskaX  = 0
MyskaY  = 0
SimulaceZastavena = 0.0

SnimekHotov = 1                         # Dulezita pomocna promenna, GLUT jinak s radosti 
                                        # zahlcuje ovladac OpenGL hroznym zpusobem

glutIdleFunc    (_IdleFunc)             # co se deje kdyz nekreslim
glutDisplayFunc (_DrawFunc)             # bude se volat kdyz chci kreslit
glutKeyboardFunc(GLUT_CBKlavesnice)     # vstup z klavesnice
glutSpecialFunc (GLUT_CBKlavesnicePlus) # vstup z  F1-F12, sipky...               
glutReshapeFunc (GLUT_CBZmenaRozmeru)   # zmena dimenzi okna
glutMouseFunc   (GLUT_CBMys)

# Menu 
menu_BodyKamera = glutCreateMenu(kontextoveMenu_KameraFokus)
item = 0
for i in XODE_ListBodies:
  glutAddMenuEntry(str( XODE_ListBodies[item].name), item)
  item += 1
  
menu_Ovladani = glutCreateMenu(kontextoveMenu_Hlavni)
glutAddMenuEntry( 'Rotace kamery - sipky', 0)
glutAddMenuEntry( 'Silove pusobeni - W,S,A,D', 0)
menu_Hlavni = glutCreateMenu(kontextoveMenu_Hlavni)
glutAddSubMenu("Kamera BODY", menu_BodyKamera)
glutAddSubMenu("Popis ovladani", menu_Ovladani)
glutAddMenuEntry('Spustit/Pozastavit simulaci', 998)
glutAddMenuEntry('Kamera globalni', 999)
glutAddMenuEntry('Ukoncit program', 1000)
glutAttachMenu(GLUT_RIGHT_BUTTON)

if len(XODE_ListGeoms) == 0:
  print '[!] Varovani: Neni co vykreslovat'
else:
  print '[i] Soubor nacten v poradku'  

# Lepsi vypsat jen pocty nalezenych prvku, delsi textovy vypis dost zpomaluje
print "bodies:", len(XODE_ListBodies)
print "geoms:", len(XODE_ListGeoms)
print "joints:", len(XODE_ListJoints)


# Tady to zustane viset dokud se program neukonci WM_CLOSE nebo WM_DESTROY :)
glutMainLoop ()

# Cistka a konec
OpenGL_UvolniDisplayListy()  # podstatne zejmena na Win9x/ME
glutDestroyMenu(menu_BodyKamera)
glutDestroyMenu(menu_Ovladani)
glutDestroyMenu(menu_Kamera)
glutDestroyWindow()  
